<?php

declare(strict_types=1);

namespace App\Repository\Users;

use App\Entity\Database\Users\Company;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;

/**
 * In case of deleting records, you must modify RefreshDatabaseCommand transaction logic
 * @extends ServiceEntityRepository<Company>
 */
class CompanyRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Company::class);
    }

    /**
     * Find company by name
     * @param string $name
     * @return Company|null
     */
    public function findByName(string $name): ?Company
    {
        return $this->findOneBy(['name' => $name]);
    }

    /**
     * Delete companies with no associated users
     * @return void
     */
    public function deleteOrphanRecords(): void
    {
        $this->getEntityManager()->createQuery('DELETE FROM App\Entity\Database\Users\Company c WHERE NOT EXISTS (SELECT 1 FROM App\Entity\Database\Users\User u WHERE u.company = c.id)')
            ->execute();
    }
}
