<?php

declare(strict_types=1);

namespace App\Repository\Users;

use App\Entity\Database\Users\User;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;

/**
 * In case of deleting records, you must modify RefreshDatabaseCommand transaction logic
 * @extends ServiceEntityRepository<User>
 */
class UserRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, User::class);
    }

    /**
     * Find users by IDs. If user not exists, will not be set in the result array.
     * @param array<int> $ids
     * @return array<User>
     */
    public function findByIds(array $ids): array
    {
        $result = [];
        $resultQ = $this->createQueryBuilder('u')
            ->where('u.id IN (:ids)')
            ->setParameter('ids', $ids)
            ->getQuery()
            ->getResult();

        foreach ($resultQ as $user) {
            $result[$user->id] = $user;
        }

        return $result;
    }
}
