<?php

declare(strict_types=1);

namespace App\Service\Remote;

use App\Entity\Remote\Brilo\Users\User;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpClient\Exception\TransportException;
use Symfony\Component\Serializer\SerializerInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

final class BriloApiUsers
{
    use BriloApiFetchTrait;

    public function __construct(
        protected readonly HttpClientInterface $httpClient,
        protected readonly LoggerInterface $logger,
        protected readonly SerializerInterface $serializer,
        protected readonly string $url,
        protected readonly int $retryCount = 3,
        protected readonly int $sleepTimeS = 1,
    ) {
    }

    /**
     * Fetch all users from the Brilo API and return them as an array of User entities.
     * @return User[]
     * @throws \Exception
     */
    public function getUsers(): array
    {
        return $this->fetchApiResponse($this->url, User::class);
    }
}
